/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.Arrays;
import java.util.List;

import javax.xml.namespace.QName;

import org.junit.Test;
import org.ow2.petals.jbi.descriptor.original.generated.LinkType;
import org.ow2.petals.microkernel.api.jbi.servicedesc.JBIServiceEndpointImpl;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint.EndpointType;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;
import com.ebmwebsourcing.easycommons.uuid.QualifiedUUIDGenerator;

/**
 * Unit tests of {@link RegistryOverlayImpl#getInternalEndpointsForInterface(QName, LinkType)}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayImpl_GetIntEpsForItf_Test extends AbstractRegistryOverlayImplTest {

    /**
     * A namespace name
     */
    private final static String A_NAMESPACE = "http://petals.ow2.org/unit-tests";

    /**
     * A service name
     */
    private final static QName SERVICE_NAME_1 = new QName(A_NAMESPACE, "service-1");

    /**
     * Another service name
     */
    private final static QName SERVICE_NAME_2 = new QName(A_NAMESPACE, "service-2");

    /**
     * A interface name
     */
    private final static QName INTERFACE_NAME_1 = new QName(A_NAMESPACE, "interface-1");

    /**
     * Another interface name
     */
    private final static QName INTERFACE_NAME_2 = new QName(A_NAMESPACE, "interface-2");

    /**
     * A local container name
     */
    private final static String LOCAL_CONTAINER_NAME = "local-container";

    /**
     * An internal endpoint name on a local container
     */
    private final static String LOCAL_INTERNAL_ENDPOINT_NAME_1;

    /**
     * Another internal endpoint name on a local container
     */
    private final static String LOCAL_INTERNAL_ENDPOINT_NAME_2;

    /**
     * Another internal endpoint name on a local container
     */
    private final static String LOCAL_INTERNAL_ENDPOINT_NAME_3;

    static {
        final QualifiedUUIDGenerator uuidGenerator = new QualifiedUUIDGenerator("petals");
        LOCAL_INTERNAL_ENDPOINT_NAME_1 = uuidGenerator.getNewID();
        LOCAL_INTERNAL_ENDPOINT_NAME_2 = uuidGenerator.getNewID();
        LOCAL_INTERNAL_ENDPOINT_NAME_3 = uuidGenerator.getNewID();
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getInternalEndpointsForInterface(QName, LinkType)} :
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>three internal endpoints are registered on a local container,</li>
     * <li>an existing interface name is provided as paramter.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error occurs</li>
     * <li>the endpoint implementing the interface is returned</li>
     * </ol>
     * </p>
     */
    @Test
    public void testToGetEndpointsOfSpecifiedItf() throws Exception {

        final String topologyName = this.getClass().getSimpleName() + "_testToGetEndpointsOfSpecifiedItf";
        final RegistryOverlayImpl localRegistryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(localRegistryOverlayImpl, "configurationService",
                RegistryOverlayImpl_GetIntEpsForItf_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), topologyName, LOCAL_CONTAINER_NAME));

        try {
            localRegistryOverlayImpl.start();

            final JBIServiceEndpointImpl localInternalServiceEndpointMock_1 = new JBIServiceEndpointImpl();
            localInternalServiceEndpointMock_1.setEndpointName(LOCAL_INTERNAL_ENDPOINT_NAME_1);
            localInternalServiceEndpointMock_1.setServiceName(SERVICE_NAME_1);
            localInternalServiceEndpointMock_1.setInterfacesName(Arrays.asList(new QName[] { INTERFACE_NAME_1 }));
            localInternalServiceEndpointMock_1.setType(EndpointType.INTERNAL);

            final JBIServiceEndpointImpl localInternalServiceEndpointMock_2 = new JBIServiceEndpointImpl();
            localInternalServiceEndpointMock_2.setEndpointName(LOCAL_INTERNAL_ENDPOINT_NAME_2);
            localInternalServiceEndpointMock_2.setServiceName(SERVICE_NAME_2);
            localInternalServiceEndpointMock_2.setInterfacesName(Arrays.asList(new QName[] { INTERFACE_NAME_2 }));
            localInternalServiceEndpointMock_2.setType(EndpointType.INTERNAL);

            final JBIServiceEndpointImpl localInternalServiceEndpointMock_3 = new JBIServiceEndpointImpl();
            localInternalServiceEndpointMock_3.setEndpointName(LOCAL_INTERNAL_ENDPOINT_NAME_3);
            localInternalServiceEndpointMock_3.setServiceName(SERVICE_NAME_2);
            localInternalServiceEndpointMock_3.setInterfacesName(Arrays.asList(new QName[] { INTERFACE_NAME_2 }));
            localInternalServiceEndpointMock_3.setType(EndpointType.INTERNAL);

            // Add endpoints
            localRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_1, LOCAL_INTERNAL_ENDPOINT_NAME_1,
                    Arrays.asList(new QName[] { INTERFACE_NAME_1 }), null, localInternalServiceEndpointMock_1, false);
            localRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_2, LOCAL_INTERNAL_ENDPOINT_NAME_2,
                    Arrays.asList(new QName[] { INTERFACE_NAME_2 }), null, localInternalServiceEndpointMock_2, false);
            localRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_2, LOCAL_INTERNAL_ENDPOINT_NAME_3,
                    Arrays.asList(new QName[] { INTERFACE_NAME_2 }), null, localInternalServiceEndpointMock_3, false);

            // Check that endpoints are registered
            final List<PetalsServiceEndpoint> registeredSvcEp = localRegistryOverlayImpl.getEndpoints();
            assertNotNull("Null list returned", registeredSvcEp);
            assertEquals("Invalid response number retrieving endpoints for service", 3, registeredSvcEp.size());

            // get all endpoints implementing a specified interface
            final PetalsServiceEndpoint[] allEndpoints = localRegistryOverlayImpl.getInternalEndpointsForInterface(
                    INTERFACE_NAME_1, null);
            assertNotNull("Null list returned", allEndpoints);
            assertEquals("Unexpected number of returned endpoints", 1, allEndpoints.length);
            final PetalsServiceEndpoint retrievedEp = allEndpoints[0];
            assertEquals("Unexpected endpoint name", LOCAL_INTERNAL_ENDPOINT_NAME_1, retrievedEp.getEndpointName());
            assertEquals("Unexpecetd service name", SERVICE_NAME_1, retrievedEp.getServiceName());

        } finally {
            localRegistryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getInternalEndpointsForInterface(QName, LinkType)} :
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>two internal endpoints are registered on a local container,</li>
     * <li>no interface name is provided as paramter.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error occurs</li>
     * <li>all internal endpoints are returned</li>
     * </ol>
     * </p>
     */
    @Test
    public void testToGetEndpointsOfAllItf() throws Exception {

        final String topologyName = this.getClass().getSimpleName() + "_testToGetEndpointsOfAllItf";
        final RegistryOverlayImpl localRegistryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(localRegistryOverlayImpl, "configurationService",
                RegistryOverlayImpl_GetIntEpsForItf_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), topologyName, LOCAL_CONTAINER_NAME));

        try {
            localRegistryOverlayImpl.start();

            final JBIServiceEndpointImpl localInternalServiceEndpointMock_1 = new JBIServiceEndpointImpl();
            localInternalServiceEndpointMock_1.setEndpointName(LOCAL_INTERNAL_ENDPOINT_NAME_1);
            localInternalServiceEndpointMock_1.setServiceName(SERVICE_NAME_1);
            localInternalServiceEndpointMock_1.setInterfacesName(Arrays.asList(new QName[] { INTERFACE_NAME_1 }));
            localInternalServiceEndpointMock_1.setType(EndpointType.INTERNAL);

            final JBIServiceEndpointImpl localInternalServiceEndpointMock_2 = new JBIServiceEndpointImpl();
            localInternalServiceEndpointMock_2.setEndpointName(LOCAL_INTERNAL_ENDPOINT_NAME_2);
            localInternalServiceEndpointMock_2.setServiceName(SERVICE_NAME_2);
            localInternalServiceEndpointMock_2.setInterfacesName(Arrays.asList(new QName[] { INTERFACE_NAME_2 }));
            localInternalServiceEndpointMock_2.setType(EndpointType.INTERNAL);

            final JBIServiceEndpointImpl localInternalServiceEndpointMock_3 = new JBIServiceEndpointImpl();
            localInternalServiceEndpointMock_3.setEndpointName(LOCAL_INTERNAL_ENDPOINT_NAME_3);
            localInternalServiceEndpointMock_3.setServiceName(SERVICE_NAME_2);
            localInternalServiceEndpointMock_3.setInterfacesName(Arrays.asList(new QName[] { INTERFACE_NAME_2 }));
            localInternalServiceEndpointMock_3.setType(EndpointType.INTERNAL);

            // Add endpoints
            localRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_1, LOCAL_INTERNAL_ENDPOINT_NAME_1,
                    Arrays.asList(new QName[] { INTERFACE_NAME_1 }), null, localInternalServiceEndpointMock_1, false);
            localRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_2, LOCAL_INTERNAL_ENDPOINT_NAME_2,
                    Arrays.asList(new QName[] { INTERFACE_NAME_2 }), null, localInternalServiceEndpointMock_2, false);
            localRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_2, LOCAL_INTERNAL_ENDPOINT_NAME_3,
                    Arrays.asList(new QName[] { INTERFACE_NAME_2 }), null, localInternalServiceEndpointMock_3, false);

            // Check that endpoints are registered
            final List<PetalsServiceEndpoint> registeredSvcEp = localRegistryOverlayImpl.getEndpoints();
            assertNotNull("Null list returned", registeredSvcEp);
            assertEquals("Invalid response number retrieving endpoints for service", 3, registeredSvcEp.size());

            // get all endpoints implementing a specified interface
            final PetalsServiceEndpoint[] allEndpoints = localRegistryOverlayImpl.getInternalEndpointsForInterface(
                    INTERFACE_NAME_2, null);
            assertNotNull("Null list returned", allEndpoints);
            assertEquals("Unexpected number of returned endpoints", 2, allEndpoints.length);

        } finally {
            localRegistryOverlayImpl.stop();
        }
    }

}
