/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.net.URL;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.junit.Test;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlReaders;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlWriters;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlWriter;
import org.ow2.petals.jbi.descriptor.original.generated.LinkType;
import org.ow2.petals.microkernel.api.jbi.servicedesc.JBIServiceEndpointImpl;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint.EndpointType;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;
import com.ebmwebsourcing.easycommons.uuid.QualifiedUUIDGenerator;

/**
 * Unit tests of {@link RegistryOverlayImpl#getInternalEndpointsForInterface(QName, LinkType)}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayImpl_GetInternalEndpointsForInterface_Test extends AbstractRegistryOverlayImplTest {

    /**
     * A namespace name
     */
    private final static String A_NAMESPACE = "http://petals.ow2.org/unit-tests";

    /**
     * A service name
     */
    private final static QName SERVICE_NAME_1 = new QName(A_NAMESPACE, "service-1");

    /**
     * Another service name
     */
    private final static QName SERVICE_NAME_2 = new QName(A_NAMESPACE, "service-2");

    /**
     * A interface name
     */
    private final static QName INTERFACE_NAME_1 = new QName(A_NAMESPACE, "interface-1");

    /**
     * Another interface name
     */
    private final static QName INTERFACE_NAME_2 = new QName(A_NAMESPACE, "interface-2");

    /**
     * A local container name
     */
    private final static String LOCAL_CONTAINER_NAME = "local-container";

    /**
     * An internal endpoint name on a local container
     */
    private final static String LOCAL_INTERNAL_ENDPOINT_NAME_1;

    /**
     * Another internal endpoint name on a local container
     */
    private final static String LOCAL_INTERNAL_ENDPOINT_NAME_2;

    /**
     * A third internal endpoint name on a local container
     */
    private final static String LOCAL_INTERNAL_ENDPOINT_NAME_3;

    static {
        final QualifiedUUIDGenerator uuidGenerator = new QualifiedUUIDGenerator("petals");
        LOCAL_INTERNAL_ENDPOINT_NAME_1 = uuidGenerator.getNewID();
        LOCAL_INTERNAL_ENDPOINT_NAME_2 = uuidGenerator.getNewID();
        LOCAL_INTERNAL_ENDPOINT_NAME_3 = uuidGenerator.getNewID();
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getInternalEndpointsForInterface(QName, LinkType)} :
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>several internal endpoints of different interfaces are registered on a local container.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error occurs</li>
     * <li>the endpoints of the given interface are retrieved with the right attributes</li>
     * </ol>
     * </p>
     */
    @Test
    public void retrieveEndpointByInterface() throws Exception {

        final String topologyName = this.getClass().getSimpleName() + "_retrieveEndpointByInterface";
        final RegistryOverlayImpl localRegistryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(localRegistryOverlayImpl, "configurationService",
                RegistryOverlayImpl_GetInternalEndpointsForInterface_Test.createCfgServiceMock(
                        PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(),
                        PETALS_REGISTRY_NODE.getHost(), String.valueOf(PETALS_REGISTRY_NODE.getPort()), topologyName,
                        LOCAL_CONTAINER_NAME));

        try {
            localRegistryOverlayImpl.start();

            final JBIServiceEndpointImpl localInternalServiceEndpointMock_1 = new JBIServiceEndpointImpl();
            localInternalServiceEndpointMock_1.setEndpointName(LOCAL_INTERNAL_ENDPOINT_NAME_1);
            localInternalServiceEndpointMock_1.setServiceName(SERVICE_NAME_1);
            localInternalServiceEndpointMock_1.setInterfacesName(Arrays.asList(new QName[] { INTERFACE_NAME_1 }));
            localInternalServiceEndpointMock_1.setType(EndpointType.INTERNAL);

            final JBIServiceEndpointImpl localInternalServiceEndpointMock_2 = new JBIServiceEndpointImpl();
            localInternalServiceEndpointMock_2.setEndpointName(LOCAL_INTERNAL_ENDPOINT_NAME_2);
            localInternalServiceEndpointMock_2.setServiceName(SERVICE_NAME_1);
            localInternalServiceEndpointMock_2.setInterfacesName(Arrays.asList(new QName[] { INTERFACE_NAME_1 }));
            localInternalServiceEndpointMock_2.setType(EndpointType.INTERNAL);

            final JBIServiceEndpointImpl localInternalServiceEndpointMock_3 = new JBIServiceEndpointImpl();
            localInternalServiceEndpointMock_3.setEndpointName(LOCAL_INTERNAL_ENDPOINT_NAME_3);
            localInternalServiceEndpointMock_3.setServiceName(SERVICE_NAME_2);
            localInternalServiceEndpointMock_3.setInterfacesName(Arrays.asList(new QName[] { INTERFACE_NAME_2 }));
            localInternalServiceEndpointMock_3.setType(EndpointType.INTERNAL);

            // Read the wsdl
            final URL wsdlUrl = Thread.currentThread().getContextClassLoader().getResource("echo.wsdl");
            assertNotNull("WSDL not found", wsdlUrl);
            final WSDL4ComplexWsdlReader wsdl4ComplexWsdlReader = WSDL4ComplexWsdlReaders.takeWSDL4ComplexWsdlReader();
            try {
                final Description wsdlDescription = wsdl4ComplexWsdlReader.read(wsdlUrl);
                wsdlDescription.addImportedDocumentsInWsdl();
                final WSDL4ComplexWsdlWriter wsdl4ComplexWsdlWriter = WSDL4ComplexWsdlWriters
                        .takeWSDL4ComplexWsdlWriter();
                try {
                    final Document document = wsdl4ComplexWsdlWriter.getDocument(wsdlDescription);

                    // Add endpoints
                    localRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_1, LOCAL_INTERNAL_ENDPOINT_NAME_1,
                            Arrays.asList(new QName[] { INTERFACE_NAME_1 }), document,
                            localInternalServiceEndpointMock_1, false);
                    localRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_1, LOCAL_INTERNAL_ENDPOINT_NAME_2,
                            Arrays.asList(new QName[] { INTERFACE_NAME_1 }), document,
                            localInternalServiceEndpointMock_2, false);
                    localRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_2, LOCAL_INTERNAL_ENDPOINT_NAME_3,
                            Arrays.asList(new QName[] { INTERFACE_NAME_2 }), document,
                            localInternalServiceEndpointMock_3, false);

                } finally {
                    WSDL4ComplexWsdlWriters.releaseWSDL4ComplexWsdlWriter(wsdl4ComplexWsdlWriter);
                }

            } finally {
                WSDL4ComplexWsdlReaders.releaseWSDL4ComplexWsdlReader(wsdl4ComplexWsdlReader);
            }

            // Check that all endpoint are correctly registered
            final List<Map<String, Object>> endpoints = localRegistryOverlayImpl.getAllEndpoints();
            assertNotNull("Null list returned", endpoints);
            assertEquals("Unexpected number of endpoints returned", 3, endpoints.size());

            // Check that all endpoints are correctly registered for each interface
            final PetalsServiceEndpoint[] service1Endpoints = localRegistryOverlayImpl.getInternalEndpointsForInterface(
                    INTERFACE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", service1Endpoints);
            assertEquals("Unexpected number of endpoints returned", 2, service1Endpoints.length);
            boolean endpoint1Found = false;
            boolean endpoint2Found = false;
            for (final PetalsServiceEndpoint service1Endpoint : service1Endpoints) {

                assertEquals(1, service1Endpoint.getInterfaces().length);
                assertEquals("Unexpected interface name", INTERFACE_NAME_1, service1Endpoint.getInterfaces()[0]);
                if (LOCAL_INTERNAL_ENDPOINT_NAME_1.equals(service1Endpoint.getEndpointName())) {
                    endpoint1Found = true;
                } else if (LOCAL_INTERNAL_ENDPOINT_NAME_2.equals(service1Endpoint.getEndpointName())) {
                    endpoint2Found = true;
                } else {
                    fail("Unexpected endpoint name: " + service1Endpoint.getEndpointName());
                }

                final Document retrievedDocument = service1Endpoint.getDescription();
                assertNotNull("WSDL description is null", retrievedDocument);
                final WSDL4ComplexWsdlReader anotherComplexWsdlReader = WSDL4ComplexWsdlReaders
                        .takeWSDL4ComplexWsdlReader();
                try {
                    final Description wsdlDescription = anotherComplexWsdlReader.read(retrievedDocument);
                    assertNotNull("EasyWSDL description is null", wsdlDescription);

                } finally {
                    WSDL4ComplexWsdlReaders.releaseWSDL4ComplexWsdlReader(anotherComplexWsdlReader);
                }
            }
            assertTrue("Endpoint 1 missing", endpoint1Found);
            assertTrue("Endpoint 2 missing", endpoint2Found);

            // Asking for the other interface name
            final PetalsServiceEndpoint[] otherServiceEndpoints = localRegistryOverlayImpl.getInternalEndpointsForInterface(
                    INTERFACE_NAME_2, LinkType.STANDARD);
            assertNotNull("Null list returned", otherServiceEndpoints);
            assertEquals("Unexpected number of endpoints returned", 1, otherServiceEndpoints.length);
            assertEquals(1, otherServiceEndpoints[0].getInterfaces().length);
            assertEquals("Unexpected interface name", INTERFACE_NAME_2, otherServiceEndpoints[0].getInterfaces()[0]);
            assertEquals("Unexpected endpoint name", LOCAL_INTERNAL_ENDPOINT_NAME_3,
                    otherServiceEndpoints[0].getEndpointName());
            final Document otherServiceDocument = otherServiceEndpoints[0].getDescription();
            assertNotNull("WSDL description is null", otherServiceDocument);
            final WSDL4ComplexWsdlReader anotherComplexWsdlReader = WSDL4ComplexWsdlReaders
                    .takeWSDL4ComplexWsdlReader();
            try {
                final Description wsdlDescription = anotherComplexWsdlReader.read(otherServiceDocument);
                assertNotNull("EasyWSDL description is null", wsdlDescription);

            } finally {
                WSDL4ComplexWsdlReaders.releaseWSDL4ComplexWsdlReader(anotherComplexWsdlReader);
            }

            // Asking for an unexisting service name
            final PetalsServiceEndpoint[] unexistingServiceEndpoints = localRegistryOverlayImpl
                    .getInternalEndpointsForService(new QName(A_NAMESPACE, "unexisting-service"), LinkType.STANDARD);
            assertNotNull("Null list returned", unexistingServiceEndpoints);
            assertEquals("Unexpected number of endpoints returned", 0, unexistingServiceEndpoints.length);

        } finally {
            localRegistryOverlayImpl.stop();
        }
    }

}
