/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.concurrent.Semaphore;
import java.util.concurrent.atomic.AtomicBoolean;

import org.junit.Test;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.exception.DynamicTopologyNotLockedByCurrentThreadException;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.exception.DynamicTopologyNotLockedException;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Unit tests of {@link RegistryOverlayImpl#lockDynamicTopology()} and
 * {@link RegistryOverlayImpl#unlockDynamicTopology()}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayImpl_LockDynamicTopology_Test extends AbstractRegistryOverlayImplTest {

    /**
     * A container name
     */
    private final static String CONTAINER_NAME_1 = "container-1";

    /**
     * Another container name
     */
    private final static String CONTAINER_NAME_2 = "container-2";

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#lockDynamicTopology()}:
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>the dynamic topology is locked by only one thread hold by only one simulated Petals node.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error occurs</li>
     * <li>the lock can be locked</li>
     * </ol>
     * </p>
     */
    @Test
    public void testLockNotAlreadyAcquired() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                RegistryOverlayImpl_LockDynamicTopology_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "testLockNotAlreadyAcquired", CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();

            final boolean isLockAcquired = registryOverlayImpl.lockDynamicTopology();
            try {
                assertTrue("The lock is not acquired", isLockAcquired);
            } finally {
                registryOverlayImpl.unlockDynamicTopology();
            }
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#unlockDynamicTopology()}:
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>the dynamic topology lock is not acquired.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>the error {@link DynamicTopologyNotLockedException} occurs</li>
     * </ol>
     * </p>
     */
    @Test(expected = DynamicTopologyNotLockedException.class)
    public void testUnlockANotLockedlock() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                RegistryOverlayImpl_LockDynamicTopology_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "testUnlockANotLockedlock", CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();

            registryOverlayImpl.unlockDynamicTopology();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#lockDynamicTopology()} and
     * {@link RegistryOverlayImpl#unlockDynamicTopology()}:
     * <ol>
     * <li>in a dedicated topology with two nodes,</li>
     * <li>the dynamic topology is already locked by a Petals node.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error occurs</li>
     * <li>the lock can not be locked on the other Petals node, except when it is released by the other thread,</li>
     * <li>the lock can be unlocked only by the thread that locked it,</li>
     * </ol>
     * </p>
     */
    @Test
    public void testLockAlreadyAcquired() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl_1 = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl_1, "configurationService",
                RegistryOverlayImpl_LockDynamicTopology_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "testLockNotAlreadyAcquired", CONTAINER_NAME_1));
        final RegistryOverlayImpl registryOverlayImpl_2 = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl_2, "configurationService",
                RegistryOverlayImpl_LockDynamicTopology_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "testLockNotAlreadyAcquired", CONTAINER_NAME_2));

        try {
            registryOverlayImpl_1.start();
            registryOverlayImpl_2.start();

            final AtomicBoolean isCorrectlyLocked = new AtomicBoolean(false);
            final AtomicBoolean isErrorOccures = new AtomicBoolean(false);

            final Semaphore lockIsLockedLock = new Semaphore(1);
            final Semaphore waitBeforeUnLockByOwnerLock = new Semaphore(1);
            final Semaphore waitBeforeToRetryLock = new Semaphore(1);

            lockIsLockedLock.acquire();
            waitBeforeToRetryLock.acquire();
            final Thread aThread = new Thread(new Runnable() {

                @Override
                public void run() {

                    try {
                        final boolean isLockLocked = registryOverlayImpl_2.lockDynamicTopology();
                        if (isLockLocked) {
                            isCorrectlyLocked.set(true);
                        }
                        waitBeforeUnLockByOwnerLock.acquire();
                        lockIsLockedLock.release();

                        waitBeforeUnLockByOwnerLock.acquire();
                        registryOverlayImpl_2.unlockDynamicTopology();
                        waitBeforeToRetryLock.release();

                    } catch (final InterruptedException e) {
                        isErrorOccures.set(true);
                        e.printStackTrace();
                    } catch (final DynamicTopologyNotLockedException e) {
                        isErrorOccures.set(true);
                        e.printStackTrace();
                    } catch (final DynamicTopologyNotLockedByCurrentThreadException e) {
                        isErrorOccures.set(true);
                        e.printStackTrace();
                    }
                }
            });

            aThread.start();
            try {
                lockIsLockedLock.acquire();
                assertTrue("Lock not correctly initialized, not locked", isCorrectlyLocked.get());

                assertFalse("The lock is acquired even if already acquired",
                        registryOverlayImpl_1.lockDynamicTopology());
                try {
                    registryOverlayImpl_1.unlockDynamicTopology();
                    fail("Unlock by a thread different from the owner succeeded");
                } catch (final DynamicTopologyNotLockedByCurrentThreadException e) {
                    // NOP: expected exception
                }
                waitBeforeUnLockByOwnerLock.release();
                waitBeforeToRetryLock.acquire();
                assertTrue("The lock is not acquired", registryOverlayImpl_1.lockDynamicTopology());

                assertFalse("An undetected error occurs, see logs", isErrorOccures.get());
            } finally {
                aThread.interrupt();
            }

        } finally {
            registryOverlayImpl_1.stop();
            registryOverlayImpl_2.stop();
        }
    }

}
